#ifndef ALLOCATOR_H
#define ALLOCATOR_H

#include <stdbool.h>
#include <stddef.h>

typedef struct Allocator Allocator;
struct Allocator {
    void* (*alloc)(Allocator* allocator, size_t size);
    void* (*realloc)(Allocator* allocator, void* data, size_t size);
};

size_t align_size(size_t size);

typedef struct {
    void* data;
    size_t index;
    size_t capacity;
} GCBuffer;

void gc_buffer_construct(GCBuffer* buffer, size_t capacity);
void gc_buffer_destroy(GCBuffer* buffer);
void gc_buffer_reset(GCBuffer* buffer);
void* gc_buffer_alloc(GCBuffer* buffer, size_t size);
void gc_buffer_resize(GCBuffer* buffer, size_t minimum_capacity);
bool gc_buffer_contains(GCBuffer* buffer, void* address);

typedef enum {
    GCBufferA,
    GCBufferB,
} SelectedGCBuffer;

typedef struct GC GC;
struct GC {
    void* (*alloc)(GC* allocator, size_t size);
    void* (*realloc)(GC* allocator, void* data, size_t size);
    GCBuffer buffer_a;
    GCBuffer buffer_b;
    SelectedGCBuffer select;
};

void gc_construct(GC* allocator, size_t start_capacity);
void gc_destroy(GC* allocator);
void* gc_alloc(GC* allocator, size_t size);
void* gc_realloc(GC* allocator, void* data, size_t size);
void gc_collect(GC* allocator, void* entries, size_t entries_size);

void gc_relocate_branches(GC* allocator, void* allocation);
void gc_relocate_branches_buffer(
    GC* allocator, void* branches, size_t branches_size
);
void gc_relocate_allocation(GC* allocator, void* allocation);
GCBuffer* gc_selected_buffer(GC* allocator);
GCBuffer* gc_other_buffer(GC* allocator);
void gc_select_other(GC* allocator);

#endif
