#include "char_reader.h"
#include "panic.h"
#include "vec.h"
#include <stddef.h>
#include <stdio.h>

void file_char_reader_construct(FileCharReader* reader, FILE* file)
{
    *reader = (FileCharReader) {
        .next = file_char_reader_next,
        .value = file_char_reader_value,
        .file = file,
        .buffer = { 0 },
    };
    string_construct(&reader->buffer);
}

void file_char_reader_destroy(FileCharReader* reader)
{
    string_destroy(&reader->buffer);
}

char file_char_reader_next(FileCharReader* reader)
{
    int read_maybe_char = fgetc(reader->file);
    if (read_maybe_char == EOF) {
        return '\0';
    }
    char read_char = (char)read_maybe_char;
    string_append_char(&reader->buffer, read_char);
    return (char)read_char;
}

StringSlice file_char_reader_value(
    const FileCharReader* reader, size_t index, size_t length
)
{
    ASSERT(index + length <= reader->buffer.length);
    return (StringSlice) {
        .data = &reader->buffer.data[index],
        .length = length,
    };
}
