#ifndef LEXER_H
#define LEXER_H

#include "char_reader.h"
#include "vec.h"
#include <stdbool.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

typedef struct {
    size_t index;
    size_t line;
    size_t col;
} Pos;

typedef struct {
    Pos pos;
    String message;
} Error;

void error_construct(Error* error, Pos pos, String message);
void error_destroy(Error* error);

typedef struct {
    Error* errors;
    size_t errors_length;
    size_t errors_capacity;
} ErrorCollector;

void error_collector_construct(ErrorCollector* collector);
void error_collector_destroy(ErrorCollector* collector);
void error_collector_add(ErrorCollector* collector, Error error);

typedef enum {
    TokenTypeEof,
    TokenTypeError,
    TokenTypeId,
    TokenTypeInt,
    TokenTypeChar,
    TokenTypeString,
    TokenTypeLParen,
    TokenTypeRParen,
    TokenTypeLBrace,
    TokenTypeRBrace,
    TokenTypeLBracket,
    TokenTypeRBracket,
} TokenType;

const char* token_type_value(TokenType type);

typedef struct {
    TokenType type;
    Pos pos;
    size_t length;
} Token;

typedef struct {
    CharReader* reader;
    ErrorCollector* errors;
    char current;
    Pos pos;
} Lexer;

Lexer lexer_create(CharReader* reader, ErrorCollector* errors);
Token lexer_next(Lexer* lexer);

Token lexer_lex_id(Lexer* lexer);
Token lexer_lex_int(Lexer* lexer);
Token lexer_lex_char(Lexer* lexer);
Token lexer_lex_string(Lexer* lexer);
Token lexer_lex_slash(Lexer* lexer);
Token lexer_lex_hashtag(Lexer* lexer);
Token lexer_skip_whitespace(Lexer* lexer);
Token lexer_token(const Lexer* lexer, TokenType type, Pos pos);
void lexer_step(Lexer* lexer);
bool lexer_done(const Lexer* lexer);

#endif
